@@ -23,6 +23,8 @@
 #ifndef __GST_I18N_APP_H__
 #define __GST_I18N_APP_H__
 
+#ifdef ENABLE_NLS
+
 #include "gettext.h" /* included with gettext distribution and copied */
 
 /* we want to use shorthand _() for translating and N_() for marking */
@@ -30,4 +32,12 @@
 #define N_(String) gettext_noop (String)
 /* FIXME: if we need it, we can add Q_ as well, like in glib */
 
+#else
+
+#define _(String) String
+#define N_(String) String
+#define ngettext(Singular,Plural,Count) ((Count>1)?Plural:Singular)
+
+#endif
+
 #endif /* __GST_I18N_APP_H__ */
