@@ -93,9 +93,6 @@ typedef enum {
 
 /**
  * GstFlowReturn:
- * @GST_FLOW_CUSTOM_SUCCESS:	 Elements can use values starting from
- *                               this to define custom success codes.
- *                               Since 0.10.7.
  * @GST_FLOW_RESEND:		 Resend buffer, possibly with new caps (not
  *                                 sent yet) (unused/unimplemented).
  * @GST_FLOW_OK:		 Data passing was ok.
@@ -107,16 +104,31 @@ typedef enum {
  *                               this error should post an error message with more
  *                               details.
  * @GST_FLOW_NOT_SUPPORTED:	 This operation is not supported.
+ * @GST_FLOW_CUSTOM_SUCCESS:	 Elements can use values starting from
+ *                               this (and higher) to define custom success
+ *                               codes. Since 0.10.7.
+ * @GST_FLOW_CUSTOM_SUCCESS_1:	 Pre-defined custom success code (define your
+ *                               custom success code to this to avoid compiler
+ *                               warnings). Since 0.10.29.
+ * @GST_FLOW_CUSTOM_SUCCESS_2:	 Pre-defined custom success code. Since 0.10.29.
  * @GST_FLOW_CUSTOM_ERROR:	 Elements can use values starting from
- *                               this to define custom error codes. Since 0.10.7.
+ *                               this (and lower) to define custom error codes.
+ *                               Since 0.10.7.
+ * @GST_FLOW_CUSTOM_ERROR_1:	 Pre-defined custom error code (define your
+ *                               custom error code to this to avoid compiler
+ *                               warnings). Since 0.10.29.
+ * @GST_FLOW_CUSTOM_ERROR_2:	 Pre-defined custom error code. Since 0.10.29.
  *
  * The result of passing data to a pad.
  *
  * Note that the custom return values should not be exposed outside of the
  * element scope and are available since 0.10.7.
  */
+/* FIXME 0.11: remove custom flow returns */
 typedef enum {
   /* custom success starts here */
+  GST_FLOW_CUSTOM_SUCCESS_2 = 102,
+  GST_FLOW_CUSTOM_SUCCESS_1 = 101,
   GST_FLOW_CUSTOM_SUCCESS = 100,
 
   /* core predefined */
@@ -132,7 +144,9 @@ typedef enum {
   GST_FLOW_NOT_SUPPORTED  = -6,
 
   /* custom error starts here */
-  GST_FLOW_CUSTOM_ERROR   = -100
+  GST_FLOW_CUSTOM_ERROR   = -100,
+  GST_FLOW_CUSTOM_ERROR_1 = -101,
+  GST_FLOW_CUSTOM_ERROR_2 = -102
 } GstFlowReturn;
 
 /**
@@ -163,6 +177,49 @@ typedef enum {
 G_CONST_RETURN gchar*	gst_flow_get_name	(GstFlowReturn ret);
 GQuark			gst_flow_to_quark	(GstFlowReturn ret);
 
+/**
+ * GstPadLinkCheck:
+ * @GST_PAD_LINK_CHECK_NOTHING: Don't check hierarchy or caps compatibility.
+ * @GST_PAD_LINK_CHECK_HIERARCHY: Check the pads have same parents/grandparents.
+ *   Could be omitted if it is already known that the two elements that own the
+ *   pads are in the same bin. 
+ * @GST_PAD_LINK_CHECK_TEMPLATE_CAPS: Check if the pads are compatible by using
+ *   their template caps. This is much faster than @GST_PAD_LINK_CHECK_CAPS, but
+ *   would be unsafe e.g. if one pad has %GST_CAPS_ANY.
+ * @GST_PAD_LINK_CHECK_CAPS: Check if the pads are compatible by comparing the
+ *   caps returned by gst_pad_get_caps().
+ *
+ * The amount of checking to be done when linking pads. @GST_PAD_LINK_CHECK_CAPS
+ * and @GST_PAD_LINK_CHECK_TEMPLATE_CAPS are mutually exclusive. If both are
+ * specified, expensive but safe @GST_PAD_LINK_CHECK_CAPS are performed.
+ *
+ * <warning><para>
+ * Only disable some of the checks if you are 100% certain you know the link
+ * will not fail because of hierarchy/caps compatibility failures. If uncertain,
+ * use the default checks (%GST_PAD_LINK_CHECK_DEFAULT) or the regular methods
+ * for linking the pads.
+ * </para></warning>
+ *
+ * Since: 0.10.30
+ */
+
+typedef enum {
+  GST_PAD_LINK_CHECK_NOTHING       = 0,
+  GST_PAD_LINK_CHECK_HIERARCHY     = 1 << 0,
+  GST_PAD_LINK_CHECK_TEMPLATE_CAPS = 1 << 1,
+  GST_PAD_LINK_CHECK_CAPS          = 1 << 2
+} GstPadLinkCheck;
+
+/**
+ * GST_PAD_LINK_CHECK_DEFAULT:
+ *
+ * The default checks done when linking pads (i.e. the ones used by 
+ * gst_pad_link()).
+ *
+ * Since: 0.10.30
+ */
+#define GST_PAD_LINK_CHECK_DEFAULT (GST_PAD_LINK_CHECK_HIERARCHY | GST_PAD_LINK_CHECK_CAPS)
+
 /**
  * GstActivateMode:
  * @GST_ACTIVATE_NONE:	  	 Pad will not handle dataflow
@@ -277,8 +334,9 @@ typedef GstFlowReturn		(*GstPadChainListFunction)	(GstPad *pad, GstBufferList *l
  * #GST_FLOW_UNEXPECTED, which corresponds to EOS. In this case @buffer does not
  * contain a valid buffer.
  *
- * The buffer size of @buffer might be smaller than @length when @offset is near
- * the end of the stream.
+ * The buffer size of @buffer will only be smaller than @length when @offset is
+ * near the end of the stream. In all other cases, the size of @buffer must be
+ * exactly the requested size.
  *
  * It is allowed to call this function with a 0 @length and valid @offset, in
  * which case @buffer will contain a 0-sized buffer and the function returns
@@ -291,7 +349,8 @@ typedef GstFlowReturn		(*GstPadChainListFunction)	(GstPad *pad, GstBufferList *l
  * optimal length is returned in @buffer. The length might depend on the value
  * of @offset.
  *
- * Returns: #GST_FLOW_OK for success
+ * Returns: #GST_FLOW_OK for success and a valid buffer in @buffer. Any other
+ * return value leaves @buffer undefined.
  */
 typedef GstFlowReturn		(*GstPadGetRangeFunction)	(GstPad *pad, guint64 offset,
 		                                                 guint length, GstBuffer **buffer);
@@ -329,7 +388,7 @@ typedef gboolean		(*GstPadCheckGetRangeFunction)	(GstPad *pad);
  *
  * The signature of the internal pad link function.
  *
- * Returns: a newly allocated #GList of pads that are linked to the given pad on
+ * Returns: (element-type Gst.Pad) (transfer container): a newly allocated #GList of pads that are linked to the given pad on
  * the inside of the parent element.
  *
  * The caller must call g_list_free() on it after use.
@@ -876,6 +935,7 @@ void			gst_pad_set_unlink_function		(GstPad *pad, GstPadUnlinkFunction unlink);
 
 gboolean                gst_pad_can_link                        (GstPad *srcpad, GstPad *sinkpad);
 GstPadLinkReturn        gst_pad_link				(GstPad *srcpad, GstPad *sinkpad);
+GstPadLinkReturn        gst_pad_link_full			(GstPad *srcpad, GstPad *sinkpad, GstPadLinkCheck flags);
 gboolean		gst_pad_unlink				(GstPad *srcpad, GstPad *sinkpad);
 gboolean		gst_pad_is_linked			(GstPad *pad);
 
@@ -955,7 +1015,7 @@ gboolean		gst_pad_query_default			(GstPad *pad, GstQuery *query);
 gboolean		gst_pad_dispatcher			(GstPad *pad, GstPadDispatcherFunction dispatch,
 								 gpointer data);
 
-#ifndef GST_DISABLE_LOADSAVE
+#if !defined(GST_DISABLE_LOADSAVE) && !defined(GST_DISABLE_DEPRECATED)
 void			gst_pad_load_and_link			(xmlNodePtr self, GstObject *parent);
 #endif
 
