@@ -115,8 +115,8 @@ gst_data_queue_class_init (GstDataQueueClass * klass)
 
   parent_class = g_type_class_peek_parent (klass);
 
-  gobject_class->set_property = GST_DEBUG_FUNCPTR (gst_data_queue_set_property);
-  gobject_class->get_property = GST_DEBUG_FUNCPTR (gst_data_queue_get_property);
+  gobject_class->set_property = gst_data_queue_set_property;
+  gobject_class->get_property = gst_data_queue_get_property;
 
   /* signals */
   /**
@@ -162,9 +162,7 @@ gst_data_queue_class_init (GstDataQueueClass * klass)
           "Current amount of data in the queue (in ns)", 0, G_MAXUINT64, 0,
           G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
-  /* set several parent class virtual functions */
-  gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_data_queue_finalize);
-
+  gobject_class->finalize = gst_data_queue_finalize;
 }
 
 static void
@@ -184,6 +182,41 @@ gst_data_queue_init (GstDataQueue * queue)
   GST_DEBUG ("initialized queue's not_empty & not_full conditions");
 }
 
+/**
+ * gst_data_queue_new_full:
+ * @checkfull: the callback used to tell if the element considers the queue full
+ * or not.
+ * @fullcallback: the callback which will be called when the queue is considered full.
+ * @emptycallback: the callback which will be called when the queue is considered empty.
+ * @checkdata: a #gpointer that will be given in the @checkfull callback.
+ *
+ * Creates a new #GstDataQueue. The difference with @gst_data_queue_new is that it will
+ * not emit the 'full' and 'empty' signals, but instead calling directly @fullcallback
+ * or @emptycallback.
+ *
+ * Returns: a new #GstDataQueue.
+ *
+ * Since: 0.10.26
+ */
+
+GstDataQueue *
+gst_data_queue_new_full (GstDataQueueCheckFullFunction checkfull,
+    GstDataQueueFullCallback fullcallback,
+    GstDataQueueEmptyCallback emptycallback, gpointer checkdata)
+{
+  GstDataQueue *ret;
+
+  g_return_val_if_fail (checkfull != NULL, NULL);
+
+  ret = g_object_newv (GST_TYPE_DATA_QUEUE, 0, NULL);
+  ret->checkfull = checkfull;
+  ret->checkdata = checkdata;
+  ret->fullcallback = fullcallback;
+  ret->emptycallback = emptycallback;
+
+  return ret;
+}
+
 /**
  * gst_data_queue_new:
  * @checkfull: the callback used to tell if the element considers the queue full
@@ -196,15 +229,7 @@ gst_data_queue_init (GstDataQueue * queue)
 GstDataQueue *
 gst_data_queue_new (GstDataQueueCheckFullFunction checkfull, gpointer checkdata)
 {
-  GstDataQueue *ret;
-
-  g_return_val_if_fail (checkfull != NULL, NULL);
-
-  ret = g_object_new (GST_TYPE_DATA_QUEUE, NULL);
-  ret->checkfull = checkfull;
-  ret->checkdata = checkdata;
-
-  return ret;
+  return gst_data_queue_new_full (checkfull, NULL, NULL, checkdata);
 }
 
 static void
@@ -269,7 +294,7 @@ gst_data_queue_locked_is_full (GstDataQueue * queue)
  * gst_data_queue_flush:
  * @queue: a #GstDataQueue.
  *
- * Flushes all the contents of the @queue. Any call to #gst_data_queue_pull and
+ * Flushes all the contents of the @queue. Any call to #gst_data_queue_push and
  * #gst_data_queue_pop will be released.
  * MT safe.
  */
@@ -308,7 +333,7 @@ gst_data_queue_is_empty (GstDataQueue * queue)
  * @queue: a #GstDataQueue.
  *
  * Queries if @queue is full. This check will be done using the
- * #GstDataQueueCheckFullCallback registered with @queue.
+ * #GstDataQueueCheckFullFunction registered with @queue.
  * MT safe.
  *
  * Returns: #TRUE if @queue is full.
@@ -383,7 +408,10 @@ gst_data_queue_push (GstDataQueue * queue, GstDataQueueItem * item)
   /* We ALWAYS need to check for queue fillness */
   if (gst_data_queue_locked_is_full (queue)) {
     GST_DATA_QUEUE_MUTEX_UNLOCK (queue);
-    g_signal_emit (G_OBJECT (queue), gst_data_queue_signals[SIGNAL_FULL], 0);
+    if (G_LIKELY (queue->fullcallback))
+      queue->fullcallback (queue, queue->checkdata);
+    else
+      g_signal_emit (queue, gst_data_queue_signals[SIGNAL_FULL], 0);
     GST_DATA_QUEUE_MUTEX_LOCK_CHECK (queue, flushing);
 
     /* signal might have removed some items */
@@ -441,7 +469,10 @@ gst_data_queue_pop (GstDataQueue * queue, GstDataQueueItem ** item)
 
   if (gst_data_queue_locked_is_empty (queue)) {
     GST_DATA_QUEUE_MUTEX_UNLOCK (queue);
-    g_signal_emit (G_OBJECT (queue), gst_data_queue_signals[SIGNAL_EMPTY], 0);
+    if (G_LIKELY (queue->emptycallback))
+      queue->emptycallback (queue, queue->checkdata);
+    else
+      g_signal_emit (queue, gst_data_queue_signals[SIGNAL_EMPTY], 0);
     GST_DATA_QUEUE_MUTEX_LOCK_CHECK (queue, flushing);
 
     while (gst_data_queue_locked_is_empty (queue)) {
