@@ -26,10 +26,11 @@
 #endif
 
 /* FIXME: hack alert */
-#ifdef WIN32
+#ifdef HAVE_WIN32
 #define DISABLE_FAULT_HANDLER
 #endif
 
+#include <stdio.h>
 #include <string.h>
 #include <signal.h>
 #ifdef HAVE_UNISTD_H
@@ -75,7 +76,7 @@ static gboolean waiting_eos = FALSE;
 /* convenience macro so we don't have to litter the code with if(!quiet) */
 #define PRINT if(!quiet)g_print
 
-#ifndef GST_DISABLE_LOADSAVE
+#if !defined(GST_DISABLE_LOADSAVE) && !defined(GST_REMOVE_DEPRECATED)
 static GstElement *
 xmllaunch_parse_cmdline (const gchar ** argv)
 {
@@ -88,8 +89,8 @@ xmllaunch_parse_cmdline (const gchar ** argv)
   gint i = 0;
 
   if (!(arg = argv[0])) {
-    g_print (_
-        ("Usage: gst-xmllaunch <file.xml> [ element.property=value ... ]\n"));
+    g_printerr ("%s",
+        _("Usage: gst-xmllaunch <file.xml> [ element.property=value ... ]\n"));
     exit (1);
   }
 
@@ -159,13 +160,14 @@ fault_handler_sighandler (int signum)
    * deadlock */
   switch (signum) {
     case SIGSEGV:
-      printf ("Caught SIGSEGV\n");
+      fprintf (stderr, "Caught SIGSEGV\n");
       break;
     case SIGQUIT:
-      printf ("Caught SIGQUIT\n");
+      if (!quiet)
+        printf ("Caught SIGQUIT\n");
       break;
     default:
-      printf ("signo:  %d\n", signum);
+      fprintf (stderr, "signo:  %d\n", signum);
       break;
   }
 
@@ -183,15 +185,16 @@ fault_handler_sigaction (int signum, siginfo_t * si, void *misc)
    * deadlock */
   switch (si->si_signo) {
     case SIGSEGV:
-      printf ("Caught SIGSEGV accessing address %p\n", si->si_addr);
+      fprintf (stderr, "Caught SIGSEGV accessing address %p\n", si->si_addr);
       break;
     case SIGQUIT:
-      printf ("Caught SIGQUIT\n");
+      if (!quiet)
+        printf ("Caught SIGQUIT\n");
       break;
     default:
-      printf ("signo:  %d\n", si->si_signo);
-      printf ("errno:  %d\n", si->si_errno);
-      printf ("code:   %d\n", si->si_code);
+      fprintf (stderr, "signo:  %d\n", si->si_signo);
+      fprintf (stderr, "errno:  %d\n", si->si_errno);
+      fprintf (stderr, "code:   %d\n", si->si_code);
       break;
   }
 
@@ -210,7 +213,8 @@ fault_spin (void)
   wait (NULL);
 
   /* FIXME how do we know if we were run by libtool? */
-  printf ("Spinning.  Please run 'gdb gst-launch %d' to continue debugging, "
+  fprintf (stderr,
+      "Spinning.  Please run 'gdb gst-launch %d' to continue debugging, "
       "Ctrl-C to quit, or Ctrl-\\ to dump core.\n", (gint) getpid ());
   while (spinning)
     g_usleep (1000000);
@@ -246,6 +250,24 @@ fault_setup (void)
 }
 #endif /* DISABLE_FAULT_HANDLER */
 
+static void
+print_error_message (GstMessage * msg)
+{
+  GError *err = NULL;
+  gchar *name, *debug = NULL;
+
+  name = gst_object_get_path_string (msg->src);
+  gst_message_parse_error (msg, &err, &debug);
+
+  g_printerr (_("ERROR: from element %s: %s\n"), name, err->message);
+  if (debug != NULL)
+    g_printerr (_("Additional debug info:\n%s\n"), debug);
+
+  g_error_free (err);
+  g_free (debug);
+  g_free (name);
+}
+
 static void
 print_tag (const GstTagList * list, const gchar * tag, gpointer unused)
 {
@@ -280,9 +302,9 @@ print_tag (const GstTagList * list, const gchar * tag, gpointer unused)
     }
 
     if (i == 0) {
-      g_print ("%16s: %s\n", gst_tag_get_nick (tag), str);
+      PRINT ("%16s: %s\n", gst_tag_get_nick (tag), str);
     } else {
-      g_print ("%16s: %s\n", "", str);
+      PRINT ("%16s: %s\n", "", str);
     }
 
     g_free (str);
@@ -294,7 +316,7 @@ print_tag (const GstTagList * list, const gchar * tag, gpointer unused)
 static void
 sigint_handler_sighandler (int signum)
 {
-  g_print ("Caught interrupt -- ");
+  PRINT ("Caught interrupt -- ");
 
   /* If we were waiting for an EOS, we still want to catch
    * the next signal to shutdown properly (and the following one
@@ -309,7 +331,7 @@ sigint_handler_sighandler (int signum)
   caught_intr = TRUE;
 }
 
-/* is called every 50 milliseconds (20 times a second), the interrupt handler
+/* is called every 250 milliseconds (4 times a second), the interrupt handler
  * will set a flag for us. We react to this by posting a message. */
 static gboolean
 check_intr (GstElement * pipeline)
@@ -318,7 +340,7 @@ check_intr (GstElement * pipeline)
     return TRUE;
   } else {
     caught_intr = FALSE;
-    g_print ("handling interrupt.\n");
+    PRINT ("handling interrupt.\n");
 
     /* post an application specific message */
     gst_element_post_message (GST_ELEMENT (pipeline),
@@ -353,16 +375,17 @@ sigint_restore (void)
   sigaction (SIGINT, &action, NULL);
 }
 
+/* FIXME 0.11: remove SIGUSR handling (also from man page) */
 static void
 play_handler (int signum)
 {
   switch (signum) {
     case SIGUSR1:
-      g_print ("Caught SIGUSR1 - Play request.\n");
+      PRINT ("Caught SIGUSR1 - Play request.\n");
       gst_element_set_state (pipeline, GST_STATE_PLAYING);
       break;
     case SIGUSR2:
-      g_print ("Caught SIGUSR2 - Stop request.\n");
+      PRINT ("Caught SIGUSR2 - Stop request.\n");
       gst_element_set_state (pipeline, GST_STATE_NULL);
       break;
   }
@@ -386,6 +409,9 @@ play_signal_setup (void)
 static EventLoopResult
 event_loop (GstElement * pipeline, gboolean blocking, GstState target_state)
 {
+#ifndef DISABLE_FAULT_HANDLER
+  gulong timeout_id;
+#endif
   GstBus *bus;
   GstMessage *message = NULL;
   EventLoopResult res = ELR_NO_ERROR;
@@ -394,7 +420,7 @@ event_loop (GstElement * pipeline, gboolean blocking, GstState target_state)
   bus = gst_element_get_bus (GST_ELEMENT (pipeline));
 
 #ifndef DISABLE_FAULT_HANDLER
-  g_timeout_add (50, (GSourceFunc) check_intr, pipeline);
+  timeout_id = g_timeout_add (250, (GSourceFunc) check_intr, pipeline);
 #endif
 
   while (TRUE) {
@@ -417,19 +443,19 @@ event_loop (GstElement * pipeline, gboolean blocking, GstState target_state)
       src_obj = GST_MESSAGE_SRC (message);
 
       if (GST_IS_ELEMENT (src_obj)) {
-        g_print (_("Got message #%u from element \"%s\" (%s): "),
+        PRINT (_("Got message #%u from element \"%s\" (%s): "),
             (guint) seqnum, GST_ELEMENT_NAME (src_obj),
             GST_MESSAGE_TYPE_NAME (message));
       } else if (GST_IS_PAD (src_obj)) {
-        g_print (_("Got message #%u from pad \"%s:%s\" (%s): "),
+        PRINT (_("Got message #%u from pad \"%s:%s\" (%s): "),
             (guint) seqnum, GST_DEBUG_PAD_NAME (src_obj),
             GST_MESSAGE_TYPE_NAME (message));
       } else if (GST_IS_OBJECT (src_obj)) {
-        g_print (_("Got message #%u from object \"%s\" (%s): "),
+        PRINT (_("Got message #%u from object \"%s\" (%s): "),
             (guint) seqnum, GST_OBJECT_NAME (src_obj),
             GST_MESSAGE_TYPE_NAME (message));
       } else {
-        g_print (_("Got message #%u (%s): "), (guint) seqnum,
+        PRINT (_("Got message #%u (%s): "), (guint) seqnum,
             GST_MESSAGE_TYPE_NAME (message));
       }
 
@@ -437,10 +463,10 @@ event_loop (GstElement * pipeline, gboolean blocking, GstState target_state)
         gchar *sstr;
 
         sstr = gst_structure_to_string (s);
-        g_print ("%s\n", sstr);
+        PRINT ("%s\n", sstr);
         g_free (sstr);
       } else {
-        g_print ("no message details\n");
+        PRINT ("no message details\n");
       }
     }
 
@@ -451,7 +477,7 @@ event_loop (GstElement * pipeline, gboolean blocking, GstState target_state)
 
         gst_message_parse_new_clock (message, &clock);
 
-        g_print ("New clock: %s\n", (clock ? GST_OBJECT_NAME (clock) : "NULL"));
+        PRINT ("New clock: %s\n", (clock ? GST_OBJECT_NAME (clock) : "NULL"));
         break;
       }
       case GST_MESSAGE_CLOCK_LOST:
@@ -459,14 +485,14 @@ event_loop (GstElement * pipeline, gboolean blocking, GstState target_state)
         /* disabled for now as it caused problems with rtspsrc. We need to fix
          * rtspsrc first, then release -good before we can reenable this again
          */
-        g_print ("Clock lost, selecting a new one\n");
+        PRINT ("Clock lost, selecting a new one\n");
         gst_element_set_state (pipeline, GST_STATE_PAUSED);
         gst_element_set_state (pipeline, GST_STATE_PLAYING);
 #endif
         break;
       case GST_MESSAGE_EOS:{
         waiting_eos = FALSE;
-        g_print (_("Got EOS from element \"%s\".\n"),
+        PRINT (_("Got EOS from element \"%s\".\n"),
             GST_MESSAGE_SRC_NAME (message));
         goto exit;
       }
@@ -475,16 +501,16 @@ event_loop (GstElement * pipeline, gboolean blocking, GstState target_state)
           GstTagList *tags;
 
           if (GST_IS_ELEMENT (GST_MESSAGE_SRC (message))) {
-            g_print (_("FOUND TAG      : found by element \"%s\".\n"),
+            PRINT (_("FOUND TAG      : found by element \"%s\".\n"),
                 GST_MESSAGE_SRC_NAME (message));
           } else if (GST_IS_PAD (GST_MESSAGE_SRC (message))) {
-            g_print (_("FOUND TAG      : found by pad \"%s:%s\".\n"),
+            PRINT (_("FOUND TAG      : found by pad \"%s:%s\".\n"),
                 GST_DEBUG_PAD_NAME (GST_MESSAGE_SRC (message)));
           } else if (GST_IS_OBJECT (GST_MESSAGE_SRC (message))) {
-            g_print (_("FOUND TAG      : found by object \"%s\".\n"),
+            PRINT (_("FOUND TAG      : found by object \"%s\".\n"),
                 GST_MESSAGE_SRC_NAME (message));
           } else {
-            g_print (_("FOUND TAG\n"));
+            PRINT (_("FOUND TAG\n"));
           }
 
           gst_message_parse_tag (message, &tags);
@@ -499,7 +525,7 @@ event_loop (GstElement * pipeline, gboolean blocking, GstState target_state)
 
         gst_message_parse_info (message, &gerror, &debug);
         if (debug) {
-          g_print (_("INFO:\n%s\n"), debug);
+          PRINT (_("INFO:\n%s\n"), debug);
         }
         g_error_free (gerror);
         g_free (debug);
@@ -516,9 +542,9 @@ event_loop (GstElement * pipeline, gboolean blocking, GstState target_state)
             GST_DEBUG_GRAPH_SHOW_ALL, "gst-launch.warning");
 
         gst_message_parse_warning (message, &gerror, &debug);
-        g_print (_("WARNING: from element %s: %s\n"), name, gerror->message);
+        PRINT (_("WARNING: from element %s: %s\n"), name, gerror->message);
         if (debug) {
-          g_print (_("Additional debug info:\n%s\n"), debug);
+          PRINT (_("Additional debug info:\n%s\n"), debug);
         }
         g_error_free (gerror);
         g_free (debug);
@@ -526,17 +552,12 @@ event_loop (GstElement * pipeline, gboolean blocking, GstState target_state)
         break;
       }
       case GST_MESSAGE_ERROR:{
-        GError *gerror;
-        gchar *debug;
-
         /* dump graph on error */
         GST_DEBUG_BIN_TO_DOT_FILE_WITH_TS (GST_BIN (pipeline),
             GST_DEBUG_GRAPH_SHOW_ALL, "gst-launch.error");
 
-        gst_message_parse_error (message, &gerror, &debug);
-        gst_object_default_error (GST_MESSAGE_SRC (message), gerror, debug);
-        g_error_free (gerror);
-        g_free (debug);
+        print_error_message (message);
+
         /* we have an error */
         res = ELR_ERROR;
         goto exit;
@@ -652,6 +673,9 @@ exit:
     if (message)
       gst_message_unref (message);
     gst_object_unref (bus);
+#ifndef DISABLE_FAULT_HANDLER
+    g_source_remove (timeout_id);
+#endif
     return res;
   }
 }
@@ -662,6 +686,7 @@ main (int argc, char *argv[])
   /* options */
   gboolean verbose = FALSE;
   gboolean no_fault = FALSE;
+  gboolean no_sigusr_handler = FALSE;
   gboolean trace = FALSE;
   gboolean eos_on_shutdown = FALSE;
   gchar *savefile = NULL;
@@ -678,12 +703,14 @@ main (int argc, char *argv[])
         N_("Output messages"), NULL},
     {"exclude", 'X', 0, G_OPTION_ARG_NONE, &exclude_args,
         N_("Do not output status information of TYPE"), N_("TYPE1,TYPE2,...")},
-#ifndef GST_DISABLE_LOADSAVE
+#if !defined(GST_DISABLE_LOADSAVE) && !defined(GST_REMOVE_DEPRECATED)
     {"output", 'o', 0, G_OPTION_ARG_STRING, &savefile,
         N_("Save xml representation of pipeline to FILE and exit"), N_("FILE")},
 #endif
     {"no-fault", 'f', 0, G_OPTION_ARG_NONE, &no_fault,
         N_("Do not install a fault handler"), NULL},
+    {"no-sigusr-handler", '\0', 0, G_OPTION_ARG_NONE, &no_sigusr_handler,
+        N_("Do not install signal handlers for SIGUSR1 and SIGUSR2"), NULL},
     {"trace", 'T', 0, G_OPTION_ARG_NONE, &trace,
         N_("Print alloc trace (if enabled at compile time)"), NULL},
     {"eos-on-shutdown", 'e', 0, G_OPTION_ARG_NONE, &eos_on_shutdown,
@@ -706,10 +733,9 @@ main (int argc, char *argv[])
   textdomain (GETTEXT_PACKAGE);
 #endif
 
-  if (!g_thread_supported ())
-    g_thread_init (NULL);
+  g_thread_init (NULL);
 
-  gst_alloc_trace_set_flags_all (GST_ALLOC_TRACE_LIVE);
+  gst_tools_set_prgname ("gst-launch");
 
 #ifndef GST_DISABLE_OPTION_PARSING
   ctx = g_option_context_new ("PIPELINE-DESCRIPTION");
@@ -717,9 +743,9 @@ main (int argc, char *argv[])
   g_option_context_add_group (ctx, gst_init_get_option_group ());
   if (!g_option_context_parse (ctx, &argc, &argv, &err)) {
     if (err)
-      g_print ("Error initializing: %s\n", GST_STR_NULL (err->message));
+      g_printerr ("Error initializing: %s\n", GST_STR_NULL (err->message));
     else
-      g_print ("Error initializing: Unknown error!\n");
+      g_printerr ("Error initializing: Unknown error!\n");
     exit (1);
   }
   g_option_context_free (ctx);
@@ -734,21 +760,29 @@ main (int argc, char *argv[])
     fault_setup ();
 
   sigint_setup ();
-  play_signal_setup ();
+
+  if (!no_sigusr_handler)
+    play_signal_setup ();
 #endif
 
   if (trace) {
     if (!gst_alloc_trace_available ()) {
       g_warning ("Trace not available (recompile with trace enabled).");
     }
+    gst_alloc_trace_set_flags_all (GST_ALLOC_TRACE_LIVE |
+        GST_ALLOC_TRACE_MEM_LIVE);
     gst_alloc_trace_print_live ();
   }
 
   /* make a null-terminated version of argv */
   argvn = g_new0 (char *, argc);
   memcpy (argvn, argv + 1, sizeof (char *) * (argc - 1));
-#ifndef GST_DISABLE_LOADSAVE
+#if !defined(GST_DISABLE_LOADSAVE) && !defined(GST_REMOVE_DEPRECATED)
   if (strstr (argv[0], "gst-xmllaunch")) {
+    /* FIXME 0.11: remove xmllaunch entirely */
+    g_warning ("gst-xmllaunch is deprecated and broken for all but the most "
+        "simple pipelines. It will most likely be removed in future. Don't "
+        "use it.\n");
     pipeline = xmllaunch_parse_cmdline ((const gchar **) argvn);
   } else
 #endif
@@ -780,8 +814,12 @@ main (int argc, char *argv[])
     g_signal_connect (pipeline, "deep-notify",
         G_CALLBACK (gst_object_default_deep_notify), exclude_list);
   }
-#ifndef GST_DISABLE_LOADSAVE
+#if !defined(GST_DISABLE_LOADSAVE) && !defined(GST_REMOVE_DEPRECATED)
   if (savefile) {
+    g_warning ("Pipeline serialization to XML is deprecated and broken for "
+        "all but the most simple pipelines. It will most likely be removed "
+        "in future. Don't use it.\n");
+
     gst_xml_write_file (GST_ELEMENT (pipeline), fopen (savefile, "w"));
   }
 #endif
@@ -846,14 +884,8 @@ main (int argc, char *argv[])
         g_printerr (_("ERROR: pipeline doesn't want to play.\n"));
         bus = gst_element_get_bus (pipeline);
         if ((err_msg = gst_bus_poll (bus, GST_MESSAGE_ERROR, 0))) {
-          GError *gerror;
-          gchar *debug;
-
-          gst_message_parse_error (err_msg, &gerror, &debug);
-          gst_object_default_error (GST_MESSAGE_SRC (err_msg), gerror, debug);
+          print_error_message (err_msg);
           gst_message_unref (err_msg);
-          g_error_free (gerror);
-          g_free (debug);
         }
         gst_object_unref (bus);
         res = -1;
@@ -880,7 +912,7 @@ main (int argc, char *argv[])
 
       diff = GST_CLOCK_DIFF (tfthen, tfnow);
 
-      g_print (_("Execution ended after %" G_GUINT64_FORMAT " ns.\n"), diff);
+      PRINT (_("Execution ended after %" G_GUINT64_FORMAT " ns.\n"), diff);
     }
 
     PRINT (_("Setting pipeline to PAUSED ...\n"));
